'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _reduce = require('lodash/reduce');

var _reduce2 = _interopRequireDefault(_reduce);

var _bundling = require('./../utils/bundling');

var _loaders = require('./loaders.config');

var _loaders2 = _interopRequireDefault(_loaders);

var _plugins = require('./plugins.config');

var _plugins2 = _interopRequireDefault(_plugins);

var _defaultConfig = require('./defaultConfig');

var _defaultConfig2 = _interopRequireDefault(_defaultConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (config) {
  var _config$context = config.context,
      context = _config$context === undefined ? _path2.default.resolve(process.cwd(), './src') : _config$context,
      entry = config.entry,
      _config$distOutput = config.distOutput,
      distOutput = _config$distOutput === undefined ? _defaultConfig2.default.distOutput : _config$distOutput,
      _config$publicPath = config.publicPath,
      publicPath = _config$publicPath === undefined ? _defaultConfig2.default.publicPath : _config$publicPath,
      _config$jsPath = config.jsPath,
      jsPath = _config$jsPath === undefined ? _defaultConfig2.default.jsPath : _config$jsPath,
      isDevEnvironment = config.isDevEnvironment,
      routes = config.routes;


  if (!entry) {
    throw new Error('No entry files are specified');
  }

  //  Extract bundles from routes
  var bundles = (0, _bundling.getBundleEntriesFromRoutes)(routes);

  return _extends({
    context: context,

    entry: _extends({}, entry, bundles, isDevEnvironment && {
      hotReload: 'webpack-hot-middleware/client?reload=true'
    }),

    output: {
      filename: !isDevEnvironment ? jsPath + '/[name].[chunkhash].bundle.js' : jsPath + '/[name].bundle.js',
      path: _path2.default.join(process.cwd(), distOutput, publicPath),
      publicPath: publicPath,
      chunkFilename: jsPath + '/[name].[chunkhash].js'
    },

    resolve: {
      alias: (0, _reduce2.default)(bundles, function (acc, location, chunkName) {
        acc[chunkName] = _path2.default.join(context, location);
        return acc;
      }, {}),
      modules: [context, _path2.default.join(process.cwd(), 'node_modules')]
    }

  }, isDevEnvironment && {
    devServer: {
      contentBase: publicPath,
      hot: true,
      inline: true,
      historyApiFallback: true
    } && {
      devtool: 'cheap-module-source-map'
    }
  }, {

    module: {
      rules: (0, _loaders2.default)(config)
    },
    plugins: (0, _plugins2.default)(config)
  });
};