import React, { Component } from 'react';
import { expect } from 'chai';
import { mount } from 'enzyme';

import { renderRoute } from './RouteHandler';

describe('RouteHandler', () => {
  describe('when the wrapped component is stateless', () => {
    it('should set initial state to defaults', () => {
      const options = {
        context: { router: { route: { location: { pathname: '/foo/1' } } } },
      };

      const props = {
        component: () => (<div>My component</div>),
        serverProps: {},
        path: '/foo/:pageId',
      };

      const WrappedComponent = renderRoute(props);
      const wrapper = mount(<WrappedComponent />, options);
      const state = wrapper.state();

      expect(state.pathname).to.equal('/foo/1');
      expect(state.getInitialProps).to.equal(false);
      expect(state.urlParams).to.eql({ pageId: '1' });
      expect(wrapper.find(props.component).length).to.equal(1);
    });
  });

  describe('when the wrapped component is stateful with getInitialProps call', () => {
    it('should set initial state to defaults and resolve data', async () => {
      const options = {
        context: { router: { route: { location: { pathname: '/foo/1' } } } },
      };

      class StatefulComponent extends Component {
        static getInitialProps = () => Promise.resolve('foo!')
        render() { return (<div>My component</div>); }
      }

      const props = {
        component: StatefulComponent,
        serverProps: {},
        path: '/foo/:pageId',
      };

      const WrappedComponent = renderRoute(props);
      const wrapper = await mount(<WrappedComponent />, options);
      const state = wrapper.state();

      expect(state.pathname).to.equal('/foo/1');
      expect(state.getInitialProps).to.equal(true);
      expect(state.urlParams).to.eql({ pageId: '1' });
      expect(state.data).to.equal('foo!');
      expect(wrapper.find(StatefulComponent).props()).to.eql({ data: 'foo!' });
    });
  });
});
