//  @flow
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { Route } from 'react-router-dom';

import BundleLoader from './BundleLoader';

import {
  requiresPrefetch,
  fetchInitialProps,
  getParamsFromUrl,
  getMetaData,
} from './../utils';

import type { IRoute } from '../../flow/IRoute';

type DefaultProps = { [string]: any };
type Props = { [string]: any };
type State = {
  pathname: string,
  data: { [string]: any },
  getInitialProps: boolean,
  urlParams: { [string]: any },
}
type Context = {
 router: { route: { location: { pathname: string } } },
}

export const renderRoute = ({
  component,
  path,
  serverProps,
  meta,
}: IRoute) => (
  class Wrapper extends Component<DefaultProps, Props, State> {
    static contextTypes = { router: PropTypes.any }
    static defaultProps = {}
    state: State;

    constructor(props: any, context: Context) {
      super(props);
      const { pathname } = context.router.route.location;

      this.state = {
        pathname,
        data: serverProps[pathname],
        getInitialProps: requiresPrefetch(component),
        urlParams: getParamsFromUrl(path, pathname),
      };
    }

    componentDidMount() {
      this.fetchClientData()
        .then(this.updateMetaData);
    }

    fetchClientData = async () => {
      const { getInitialProps, pathname, urlParams } = this.state;
      const data = serverProps[pathname];

      if (!data && getInitialProps) {
        const clientData = await fetchInitialProps(component.getInitialProps, urlParams);
        this.setState({ data: clientData });
      }
    }

    //  Update meta with side-effects
    updateMetaData = () => {
      const { urlParams } = this.state;
      const { title, description } = getMetaData({ meta }, urlParams);
      const metaDescription = document.querySelectorAll('meta[name="description"]')[0];

      document.title = title;
      metaDescription.setAttribute('value', description);
    }

    render() {
      const { data, getInitialProps } = this.state;
      const WrappedRoute = component;

      if (!data && getInitialProps) {
        return null;
      }

      return (<WrappedRoute data={data} />);
    }
  }
);

export default (props: any) => {
  const { component, bundle, ...rest } = props;

  //  If 'bundle' is defined in the current
  //  route, then we should attempt to lazy load it
  return (typeof component !== 'undefined')
    ? <Route
      {...props}
      component={renderRoute(props)}
    />
    : <Route
      {...rest}
      render={() => <BundleLoader {...rest} bundle={bundle} />}
    />;
};
