//  @flow
import fs from 'fs';
import path from 'path';
import type { $Request, $Response } from 'express';

import type { IConfig } from '../../../flow/IConfig';
import type { IApp } from '../../../flow/IApp';

const resolveHtml = (fileLoader, indexPath) => (
  new Promise((resolve, reject) => (
    fileLoader.readFile(indexPath, (err, fileContents) => {
      let templateString;
      try {
        templateString = fileContents.toString();
      } catch (error) {
        reject('Compilation in progress... Hold tight!');
      } finally {
        resolve(templateString);
      }
    })
  ))
);

export default (app: IApp, config: IConfig) => async (
  req: $Request,
  res: $Response,
  next: Function,
) => {
  const _app = app;
  const {
    isDevEnvironment,
  } = config;

  const indexPath = (isDevEnvironment)
    ? path.resolve(process.cwd(), 'public/index.html')
    : path.resolve(process.cwd(), './public/index.html');

  const fileLoader = (isDevEnvironment)
    ? app.locals.compiler.outputFileSystem : fs;

  _app.locals.html = (app.locals && app.locals.html)
    ? app.locals.html
    : await resolveHtml(fileLoader, indexPath);

  res.set('content-type', 'text/html');
  next();
};
