//  @flow
import webpack from 'webpack';
import combineLoaders from 'webpack-combine-loaders';
import ExtractTextPlugin from 'extract-text-webpack-plugin';
import autoprefixer from 'autoprefixer';
import postcssImport from 'postcss-import';
import postcssCssNext from 'postcss-cssnext';

import defaultConfig from './defaultConfig';

import type { IConfig } from '../../flow/IConfig';

export default ({
  localIdentName = defaultConfig.localIdentName,
}: IConfig) => {
  const cssLoaderConfig = [
    {
      loader: 'css-loader',
      options: {
        modules: true,
        importLoaders: 1,
        localIdentName,
      },
    },
    {
      loader: 'postcss-loader',
      options: {
        plugins: [
          postcssImport({
            addDependencyTo: webpack,
          }),
          postcssCssNext(),
        ],
      },
    },
  ];

  return [
    {
      test: /\.js?$/,
      exclude: /node_modules/,
      loader: 'babel-loader',
      options: {
        babelrc: false,
        presets: [
          'es2015',
          'react',
          'stage-0',
        ],
        plugins: [
          'transform-regenerator',
        ],
      },
    },
    {
      test: /\.css?$/,
      exclude: /node_modules/,
      loader: ExtractTextPlugin.extract(cssLoaderConfig),
    },
    {
      test: /\.(png|svg)?$/,
      exclude: /node_modules/,
      use: {
        loader: 'file-loader',
        options: {
          name: '[name].[hash].[ext]',
          outputPath: '/images',
        },
      },
    },
  ];
};
